#include <windows.h>	
#include <string.h>
#include "c:\coding\nick\vc\generate.cpp"
#include <gl\gl.h>		
#include <gl\glu.h>		
#include <gl\glaux.h>	

static HGLRC hRC;   
static HDC hDC;  

int wWidth = 640, wHeight = 480; // Original width/height of the window

void ResizeScene(GLsizei Width, GLsizei Height), DrawScene(), InitOpenGL(), SetOpenGLOptions();
LRESULT CALLBACK WndProc(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam);

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG			msg;		
	WNDCLASS	wc;			
	HWND		hWnd;		
	char ClassName[] = "OpenGL Window Class";
	char AppName[] = "Simple OpenGL demonstration";		
	
	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wc.lpfnWndProc		= (WNDPROC) WndProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 0;
	wc.hInstance		= hInstance;
	wc.hIcon			= NULL;
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground	= NULL;
	wc.lpszMenuName		= NULL;
	wc.lpszClassName	= ClassName;

	if(!RegisterClass(&wc))
	{
		MessageBox(0,"The window class could not be registered!", "Window Creation Error", MB_ICONSTOP);
		return 0;
	}

	hWnd = CreateWindow(ClassName, AppName,	WS_OVERLAPPEDWINDOW |  WS_CLIPCHILDREN | WS_CLIPSIBLINGS, 0, 0, 
                		wWidth, wHeight, NULL, NULL, hInstance, NULL);

	if(!hWnd)
	{
		MessageBox(0,"Couldn't create window!", "Window Creation Error", MB_ICONSTOP);
		return 0;
	}

//  Comment this in if you want to use it, as I said it's out because of compatibility.
/*	
	DEVMODE dmScreenSettings;											

	memset(&dmScreenSettings, 0, sizeof(DEVMODE));
	dmScreenSettings.dmSize			= sizeof(DEVMODE);
	dmScreenSettings.dmPelsWidth	= wWidth;								
	dmScreenSettings.dmPelsHeight	= wHeight;
	dmScreenSettings.dmFields		= DM_PELSWIDTH | DM_PELSHEIGHT;
	ChangeDisplaySettings(&dmScreenSettings, CDS_FULLSCREEN);
*/


	ShowWindow(hWnd, SW_SHOW);
	UpdateWindow(hWnd);
	wglMakeCurrent(hDC, hRC);

	while (msg.message != WM_QUIT)
	{
		if(PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if(GetMessage(&msg, NULL, 0, 0))
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
		}	
		DrawScene(); 
		SwapBuffers(hDC);
	}
	return 0;
}


void InitOpenGL()
{
	SetOpenGLOptions();
	// set projection matrix options...
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();							
	gluPerspective(45.0f, (GLfloat) wWidth / (GLfloat) wHeight, 0.1f, 100.0f);	
	// clear the model view matrix.
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	return;
}

void SetOpenGLOptions()	
{	
	// Set our options.
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);		
	glClearDepth(1.0);							
	glDepthFunc(GL_LESS);						
	glEnable(GL_DEPTH_TEST);					
	glShadeModel(GL_SMOOTH);					
	return;
}

void ResizeScene(GLsizei Width, GLsizei Height)
{
	if (!Height) return;
	glViewport(0, 0, Width, Height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f, (GLfloat) Width / (GLfloat) Height, 0.1f, 100.0f);
	glMatrixMode(GL_MODELVIEW);
}

void DrawScene()
{
	// does nothing.
	glLoadIdentity();
	glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
}

void SetupScreenDetails(HWND hWnd)
{
	PIXELFORMATDESCRIPTOR PFD;
	memset(&PFD, 0, sizeof(PFD));
	PFD.nSize = sizeof(PFD);
	PFD.nVersion = 1;
	PFD.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
	PFD.iPixelType = PFD_TYPE_RGBA;
	PFD.cColorBits = 16; // whatever
	PFD.cDepthBits = 16;
	PFD.dwLayerMask = PFD_MAIN_PLANE;   
	
	GLuint PixelFormat;
	hDC = GetDC(hWnd); 
	PixelFormat = ChoosePixelFormat(hDC, &PFD);
	if(!PixelFormat)
	{   
		MessageBox(NULL, "Could not find suitable pixel format",
                          "Pixel Format Error", MB_OK);
        PostQuitMessage(0);
	}
    if(!SetPixelFormat(hDC, PixelFormat, &PFD)) 
    {
        MessageBox(NULL, "Can't set the pixel format",
                        "Pixel Format Error", MB_OK);
        PostQuitMessage(0);
    }
    hRC = wglCreateContext(hDC); 
    if(!hRC)
    {
        MessageBox(NULL, "Can't create render context.", "Render Context Error", MB_OK);
        PostQuitMessage(0);
    }
    if(!wglMakeCurrent(hDC, hRC))
    {
        MessageBox(NULL, "Can't set active render context.", "Render Context Error", MB_OK);
        PostQuitMessage(0);
    }
	return;
}


LRESULT CALLBACK WndProc(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch (Message)		
	{
		case WM_CREATE:
			SetupScreenDetails(hWnd);
			InitOpenGL();
		break;
		case WM_DESTROY:
		case WM_CLOSE:
		case WM_QUIT:			
			ChangeDisplaySettings(NULL, 0);
			wglMakeCurrent(hDC, NULL);
			wglDeleteContext(hRC);
			ReleaseDC(hWnd,hDC);
			PostQuitMessage(0);
		break;

		case WM_KEYDOWN:
			PostMessage(hWnd, WM_DESTROY, 0, 0);				
		break;

		case WM_SIZE:
			ResizeScene(LOWORD(lParam),HIWORD(lParam));
		break;
	}
	return DefWindowProc(hWnd, Message, wParam, lParam);
}
